/**
 * UBC CPSC 314, Vjan2015
 * Assignment 2 Template
 */

var scene = new THREE.Scene();

// ASSIGNMENT-SPECIFIC API EXTENSION
THREE.Object3D.prototype.setMatrix = function(a) {
  this.matrix=a;
  this.matrix.decompose(this.position,this.quaternion,this.scale);
}

// SETUP RENDERER
var renderer = new THREE.WebGLRenderer();
renderer.setClearColor(0xffffff); // white background colour
document.body.appendChild(renderer.domElement);

// SETUP CAMERA
var camera = new THREE.PerspectiveCamera(30, 1, 0.1, 1000); // view angle, aspect ratio, near, far
camera.position.set(10,15,40);
camera.lookAt(scene.position); 
scene.add(camera);

// SETUP ORBIT CONTROL OF THE CAMERA
var controls = new THREE.OrbitControls(camera);
controls.damping = 0.2;

// ADAPT TO WINDOW RESIZE
function resize() {
  renderer.setSize(window.innerWidth, window.innerHeight);
  camera.aspect = window.innerWidth / window.innerHeight;
  camera.updateProjectionMatrix();
}

window.addEventListener('resize', resize);
resize();

// FLOOR WITH CHECKERBOARD 
var floorTexture = new THREE.ImageUtils.loadTexture('images/checkerboard.jpg');
floorTexture.wrapS = floorTexture.wrapT = THREE.RepeatWrapping;
floorTexture.repeat.set(4, 4);

var floorMaterial = new THREE.MeshBasicMaterial({ map: floorTexture, side: THREE.DoubleSide });
var floorGeometry = new THREE.PlaneBufferGeometry(30, 30);
var floor = new THREE.Mesh(floorGeometry, floorMaterial);
floor.position.y = -0.1;
floor.rotation.x = Math.PI / 2;
scene.add(floor);

// UNIFORMS
var gemPosition = {type: 'v3', value: new THREE.Vector3(0,5,10)};
var gemRadius = {type: 'f', value: 1.0};

// MATERIALS
var normalMaterial = new THREE.MeshNormalMaterial();
var redMaterial = new THREE.MeshBasicMaterial({color: 0xff0000});
var gemMaterial = new THREE.ShaderMaterial({
   uniforms: {
    gemRadius : gemRadius,
  },
});

// LOAD SHADERS
var shaderFiles = [
  'glsl/gem.vs.glsl',
  'glsl/gem.fs.glsl'
];

new THREE.SourceLoader().load(shaderFiles, function(shaders) {
  gemMaterial.vertexShader = shaders['glsl/gem.vs.glsl'];
  gemMaterial.fragmentShader = shaders['glsl/gem.fs.glsl'];
})

// GEOMETRY
var parallelepiped = new THREE.BoxGeometry(4, 6, 4); // centered on origin
var sphere = new THREE.SphereGeometry(1, 32, 32); // centered on origin

var elipse = new THREE.SphereGeometry(1, 64, 64); // placed with lowest y point on origin
for (var i = 0; i < elipse.vertices.length; i++)
    elipse.vertices[i].y = (elipse.vertices[i].y + 1) * 1.5;

var thinCylinder = new THREE.CylinderGeometry(.1, .1, 2, 16); // placed with lowest y point on origin
for (var i = 0; i < thinCylinder.vertices.length; i++)
    thinCylinder.vertices[i].y += 1;

// STATIC MATRICES
var torsoMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,3, 0,0,1,0, 0,0,0,1);
var headMatrix = new THREE.Matrix4().set(1,0,0,0, 0,1,0,3, 0,0,1,0, 0,0,0,1);
var headtorsoMatrix = new THREE.Matrix4().multiplyMatrices(torsoMatrix, headMatrix);

// CREATE GEOMETRY
var gem = new THREE.Mesh(sphere, gemMaterial);
scene.add(gem);

var torso = new THREE.Mesh(parallelepiped, normalMaterial);
torso.setMatrix(torsoMatrix)
scene.add(torso);

var head = new THREE.Mesh(elipse, normalMaterial);
head.setMatrix(headtorsoMatrix);
scene.add(head);

// complete body here

var leftBeam = new THREE.Mesh(thinCylinder, redMaterial);
scene.add(leftBeam);

var rightBeam = new THREE.Mesh(thinCylinder, redMaterial);
scene.add(rightBeam);

// MOVE BODY
var clock = new THREE.Clock(true);
function updateBody() {
  var t = clock.getElapsedTime(); // current time

  // move elements here

  gem.setMatrix(new THREE.Matrix4().set(
      1,0,0,gemPosition.value.x,
      0,1,0,gemPosition.value.y,
      0,0,1,gemPosition.value.z,
      0,0,0,1
  ));
}

// LISTEN TO KEYBOARD
var keyboard = new THREEx.KeyboardState();
function checkKeyboard() {
  if (keyboard.pressed("W"))
    gemPosition.value.z -= 0.1;
  else if (keyboard.pressed("S"))
    gemPosition.value.z += 0.1;

  if (keyboard.pressed("A"))
    gemPosition.value.x -= 0.1;
  else if (keyboard.pressed("D"))
    gemPosition.value.x += 0.1;

  if (keyboard.pressed("R"))
    gemRadius.value += 0.1;
  else if (keyboard.pressed("F"))
    gemRadius.value -= 0.1;

  gemMaterial.needsUpdate = true; // Tells three.js that some uniforms might have changed
}

// SETUP UPDATE CALL-BACK
function update() {
  checkKeyboard();
  updateBody();

  requestAnimationFrame(update);
  renderer.render(scene, camera);
}

update();